@@ -1,5 +1,5 @@
 /*
- * $Id: enumerate.c,v 1.9.2.5 2002/03/12 15:09:20 srittau Exp $
+ * $Id: enumerate.c,v 1.23.2.1 2003/02/04 19:10:30 didg Exp $
  *
  * Copyright (c) 1990,1993 Regents of The University of Michigan.
  * All Rights Reserved.  See COPYRIGHT.
@@ -14,8 +14,8 @@
 #include <string.h>
 #include <dirent.h>
 #include <errno.h>
-#include <syslog.h>
 
+#include <atalk/logger.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
@@ -34,11 +34,6 @@
 #include "file.h"
 #include "filedir.h"
 
-/* check for mtab DID code */
-#ifdef DID_MTAB
-#include "parse_mtab.h"
-#endif /* DID_MTAB */
-
 #define min(a,b)	((a)<(b)?(a):(b))
 
 struct dir *
@@ -59,7 +54,7 @@ struct stat *st;
 #endif /* USE_LASTDID */
 
     if ((cdir = dirnew(namlen + 1)) == NULL) {
-        syslog( LOG_ERR, "adddir: malloc: %s", strerror(errno) );
+        LOG(log_error, logtype_afpd, "adddir: malloc: %s", strerror(errno) );
         return NULL;
     }
     strcpy( cdir->d_name, name );
@@ -85,7 +80,7 @@ struct stat *st;
     if (cdir->d_did == CNID_INVALID) {
         switch (errno) {
         case CNID_ERR_PARAM:
-            syslog(LOG_ERR, "adddir: Incorrect parameters passed to cnid_add");
+            LOG(log_error, logtype_afpd, "adddir: Incorrect parameters passed to cnid_add");
             return NULL;
         case CNID_ERR_PATH:
         case CNID_ERR_DB:
@@ -101,13 +96,8 @@ struct stat *st;
         cdir->d_did = htonl( vol->v_lastdid++ );
 #else /* USE_LASTDID */
         lstp = lstat(upath, &lst) < 0 ? st : &lst;
-#ifdef DID_MTAB
-        /* mtab way of doing DIDs */
-        cdir->d_did = htonl( afpd_st_cnid ( lstp ) );
-#else /* DID_MTAB */
         /* the old way of doing DIDs (default) */
         cdir->d_did = htonl( CNID(lstp, 0) );
-#endif /* DID_MTAB */
 #endif /* USE_LASTDID */
     }
 
@@ -115,16 +105,33 @@ struct stat *st;
 #ifndef CNID_DB
         if (edir->d_name) {
             if (strcmp(edir->d_name, cdir->d_name)) {
-                syslog(LOG_INFO, "WARNING: DID conflict for '%s' and '%s'. Are these the same file?", edir->d_name, cdir->d_name);
+                LOG(log_info, logtype_afpd, "WARNING: DID conflict for '%s' and '%s'. Are these the same file?", edir->d_name, cdir->d_name);
             }
             free(cdir->d_name);
             free(cdir);
             return edir;
         }
 #endif /* CNID_DB */
+        /* it's not possible with LASTDID
+           for CNID:
+           - someone else have moved the directory.
+           - it's a symlink inside the share.
+           - it's an ID reused, the old directory was deleted but not
+             the cnid record and the server reused the inode for 
+             the new dir.
+           for HASH (we should get ride of HASH) 
+           - someone else have moved the directory.
+           - it's an ID reused as above
+           - it's a hash duplicate and we are in big trouble
+        */
         edir->d_name = cdir->d_name;
         free(cdir);
         cdir = edir;
+        if (!cdir->d_parent || cdir->d_parent == dir) 
+            return cdir;
+            
+        /* the old was not in the same folder, remove and add to the new */
+    	dirchildremove(cdir->d_parent, cdir);
     }
 
     /* parent/child directories */
@@ -165,7 +172,7 @@ int		ibuflen, *rbuflen;
 
     if ( sd.sd_buflen == 0 ) {
         if (( sd.sd_buf = (char *)malloc( SDBUFBRK )) == NULL ) {
-            syslog( LOG_ERR, "afp_enumerate: malloc: %s", strerror(errno) );
+            LOG(log_error, logtype_afpd, "afp_enumerate: malloc: %s", strerror(errno) );
             *rbuflen = 0;
             return AFPERR_MISC;
         }
@@ -185,7 +192,7 @@ int		ibuflen, *rbuflen;
     memcpy( &did, ibuf, sizeof( did ));
     ibuf += sizeof( did );
 
-    if (( dir = dirsearch( vol, did )) == NULL ) {
+    if (( dir = dirlookup( vol, did )) == NULL ) {
         *rbuflen = 0;
         return( AFPERR_NODIR );
     }
@@ -266,8 +273,8 @@ int		ibuflen, *rbuflen;
                 start = sd.sd_buf;
                 if ((buf = (char *) realloc( sd.sd_buf, sd.sd_buflen +
                                              SDBUFBRK )) == NULL ) {
-                    syslog( LOG_ERR, "afp_enumerate: realloc: %s",
-                            strerror(errno) );
+                    LOG(log_error, logtype_afpd, "afp_enumerate: realloc: %s",
+                        strerror(errno) );
                     closedir(dp);
                     *rbuflen = 0;
                     return AFPERR_MISC;
@@ -324,9 +331,23 @@ int		ibuflen, *rbuflen;
         start = sd.sd_last;
         sd.sd_last++;
 
-        if ( stat( sd.sd_last, &st ) < 0 ) {
-            syslog( LOG_DEBUG, "afp_enumerate: stat %s: %s",
-                    sd.sd_last, strerror(errno) );
+        if (*sd.sd_last == 0) {
+            /* stat() already failed on this one */
+            sd.sd_last += len + 1;
+            continue;
+        }
+
+        if (stat( sd.sd_last, &st ) < 0 ) {
+            /*
+             * Somebody else plays with the dir, well it can be us with 
+            * "Empty Trash..."
+            */
+
+            /* so the next time it won't try to stat it again
+             * another solution would be to invalidate the cache with 
+             * sd.sd_did = -1 but if it's not ENOENT error it will start again
+             */
+            *sd.sd_last = 0;
             sd.sd_last += len + 1;
             continue;
         }
@@ -432,35 +453,3 @@ int		ibuflen, *rbuflen;
 }
 
 
-/* why is this here? well, FPCatSearch is essentially an FPEnumerate
- * with filters. */
-int afp_catsearch(AFPObj *obj, char *ibuf, int ibuflen,
-                  char *rbuf, int *rbuflen)
-{
-    struct vol *vol;
-    u_int16_t   vid;
-
-    ibuf += 2;
-    memcpy(&vid, ibuf, sizeof(vid));
-    ibuf += sizeof(vid);
-
-    *rbuflen = 0;
-    if ((vol = getvolbyvid(vid)) == NULL)
-        return AFPERR_PARAM;
-
-    /* the ritual:
-     * do a breadth-first search of directories:
-     *   lookup did/name info.
-     *   add to result if match
-     *   check to see if we've exceeded our timelimit
-     *     if yes, return current position
-     *     if not, continue
-     * 
-     *   we keep a copy of our current position in struct vol.
-     *   if the next catsearch request for that volume isn't at
-     *   at the current position, bail and return catchanged.
-     */
-
-    /* eof when done */
-    return AFPERR_EOF;
-}
