@@ -1,4 +1,6 @@
 /*
+ * $Id: atp_open.c,v 1.5 2001/08/15 02:17:57 srittau Exp $
+ *
  * Copyright (c) 1990,1991 Regents of The University of Michigan.
  * All Rights Reserved.
  *
@@ -21,51 +23,54 @@
  *	netatalk@itd.umich.edu
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <stdlib.h>
+#include <string.h>
 #include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/time.h>
 #include <sys/param.h>
+#include <sys/socket.h>
+
 #include <netatalk/at.h>
+#include <netatalk/endian.h>
+
+#include <atalk/netddp.h>
+#include <atalk/ddp.h>
 #include <atalk/atp.h>
 
-ATP atp_open( port )
-    u_char		port;
+#include "atp_internals.h"
+
+ATP atp_open(u_int8_t port, const struct at_addr *saddr)
 {
+    struct sockaddr_at  addr;
     int			s;
     ATP			atp;
     struct timeval	tv;
     int			pid;
-    int			nlen;
 
-#ifdef EBUG
+#ifdef DEBUG
     printf( "<%d> atp_open\n", getpid());
-#endif
+#endif /* DEBUG */
 
-    if (( s = socket( AF_APPLETALK, SOCK_DGRAM, 0 )) < 0 ) {
-	return 0;
-    }
+    memset(&addr, 0, sizeof(addr));
+    addr.sat_port = port;
+    if (saddr) 
+      memcpy(&addr.sat_addr, saddr, sizeof(struct at_addr));
+    if ((s = netddp_open(&addr, NULL)) < 0)
+        return NULL;
 
-    if (( atp = (ATP) alloc_buf()) == NULL ) {
-	return 0;
+    if (( atp = (ATP) atp_alloc_buf()) == NULL ) {
+        netddp_close(s);
+	return NULL;
     }
 
     /* initialize the atp handle */
-    bzero( (char *) atp, sizeof( struct atp_handle ));
-    atp->atph_saddr.sat_family = AF_APPLETALK;
-    atp->atph_saddr.sat_addr.s_net = ATADDR_ANYNET;
-    atp->atph_saddr.sat_addr.s_node = ATADDR_ANYNODE;
-    atp->atph_saddr.sat_port = ( port == 0 ? ATADDR_ANYPORT : port );
+    memset(atp, 0, sizeof( struct atp_handle ));
+    memcpy(&atp->atph_saddr, &addr, sizeof(addr));
 
-    if ( bind( s, (struct sockaddr *) &atp->atph_saddr,
-	    sizeof( struct sockaddr_at )) != 0 ) {
-	free_buf( (struct atp_buf *)atp );
-	return NULL;
-    }
-    /* get the real address from the kernel */
-    nlen = sizeof( struct sockaddr_at);
-    if ( getsockname( s, (struct sockaddr *) &atp->atph_saddr, &nlen ) != 0 ) {
-	return NULL;
-    }
     atp->atph_socket = s;
     atp->atph_reqto = -1;
     gettimeofday( &tv, (struct timezone *) 0 );
@@ -74,7 +79,7 @@ ATP atp_open( port )
 
 #ifdef EBUG
 srandom( tv.tv_sec );
-#endif
+#endif /* EBUG */
 
     return atp;
 }
